@@ -4,59 +4,27 @@
 # TODO: Get most (or all) of these from rocm_smi.h to avoid mismatches and redundancy
 
 from __future__ import print_function
-import ctypes.util
 from ctypes import *
 from enum import Enum
 
-import os
+import sys
 
-# Use ROCm installation path if running from standard installation
-# With File Reorg rsmiBindings.py will be installed in  /opt/rocm/libexec/rocm_smi.
-# relative path changed accordingly.
-# if ROCM_SMI_LIB_PATH is set, we can load 'librocm_smi64.so' from that location
-#
-# Library load is wrapped in a function so prints can be hidden for PRINT_JSON mode.
-path_librocm = str()
-def initRsmiBindings(silent=False):
-    def print_silent(*args):
-        if not silent:
-            print(args)
-
-    rocm_smi_lib_path = os.getenv('ROCM_SMI_LIB_PATH')
-    if (rocm_smi_lib_path != None):
-        path_librocm = rocm_smi_lib_path
-    else:
-        path_librocm = os.path.dirname(os.path.realpath(__file__)) + '/../../lib/librocm_smi64.so.7'
-
-    if not os.path.isfile(path_librocm):
-        print_silent('Unable to find %s . Trying /opt/rocm*' % path_librocm)
-        for root, dirs, files in os.walk('/opt', followlinks=True):
-            if 'librocm_smi64.so.7' in files:
-                path_librocm = os.path.join(os.path.realpath(root), 'librocm_smi64.so.7')
-        if os.path.isfile(path_librocm):
-            print_silent('Using lib from %s' % path_librocm)
-        else:
-            print('Unable to find librocm_smi64.so.7')
-
-    # ----------> TODO: Support static libs as well as SO
-    try:
-        cdll.LoadLibrary(path_librocm)
-        return CDLL(path_librocm)
-    except OSError:
-        print('Unable to load the rocm_smi library.\n'\
-              'Set LD_LIBRARY_PATH to the folder containing librocm_smi64.so.7\n'\
-              '{0}Please refer to https://github.com/'\
-              'RadeonOpenCompute/rocm_smi_lib for the installation guide.{1}'\
-              .format('\33[33m', '\033[0m'))
+if 'sphinx' in sys.modules:
+    path_librocm = str()
+    def initRsmiBindings(silent=False):
+        # Empty function for document generation
         exit()
 
+    SMI_HASH = '@PKG_VERSION_HASH@'
+else:
+    from rsmiBindingsInit import *
+
+
 # Device ID
 dv_id = c_uint64()
 # GPU ID
 gpu_id = c_uint32(0)
 
-SMI_HASH = '8e78352'
-
 
 # Policy enums
 RSMI_MAX_NUM_FREQUENCIES = 33
@@ -99,11 +67,11 @@ rsmi_status_verbose_err_out = {
     rsmi_status_t.RSMI_STATUS_OUT_OF_RESOURCES: 'Unable to acquire memory or other resource',
     rsmi_status_t.RSMI_STATUS_INTERNAL_EXCEPTION: 'An internal exception was caught',
     rsmi_status_t.RSMI_STATUS_INPUT_OUT_OF_BOUNDS: 'Provided input is out of allowable or safe range',
-    rsmi_status_t.RSMI_INITIALIZATION_ERROR: 'Error occured during rsmi initialization',
+    rsmi_status_t.RSMI_INITIALIZATION_ERROR: 'Error occurred during rsmi initialization',
     rsmi_status_t.RSMI_STATUS_NOT_YET_IMPLEMENTED: 'Requested function is not implemented on this setup',
     rsmi_status_t.RSMI_STATUS_NOT_FOUND: 'Item searched for but not found',
     rsmi_status_t.RSMI_STATUS_INSUFFICIENT_SIZE: 'Insufficient resources available',
-    rsmi_status_t.RSMI_STATUS_INTERRUPT: 'Interrupt occured during execution',
+    rsmi_status_t.RSMI_STATUS_INTERRUPT: 'Interrupt occurred during execution',
     rsmi_status_t.RSMI_STATUS_UNEXPECTED_SIZE: 'Unexpected amount of data read',
     rsmi_status_t.RSMI_STATUS_NO_DATA: 'No data found for the given input',
     rsmi_status_t.RSMI_STATUS_UNEXPECTED_DATA: 'Unexpected data received',
@@ -134,16 +102,18 @@ class rsmi_dev_perf_level_t(c_int):
     RSMI_DEV_PERF_LEVEL_UNKNOWN = 0x100
 
 
-notification_type_names = ['VM_FAULT', 'THERMAL_THROTTLE', 'GPU_RESET']
+notification_type_names = ['VM_FAULT', 'THERMAL_THROTTLE', 'GPU_PRE_RESET', 'GPU_POST_RESET', 'RING_HANG']
 
 
 class rsmi_evt_notification_type_t(c_int):
-    RSMI_EVT_NOTIF_VMFAULT = 0
+    RSMI_EVT_NOTIF_NONE = 0
+    RSMI_EVT_NOTIF_VMFAULT = 1
     RSMI_EVT_NOTIF_FIRST = RSMI_EVT_NOTIF_VMFAULT
-    RSMI_EVT_NOTIF_THERMAL_THROTTLE = 1
-    RSMI_EVT_NOTIF_GPU_PRE_RESET = 2
-    RSMI_EVT_NOTIF_GPU_POST_RESET = 3
-    RSMI_EVT_NOTIF_LAST = RSMI_EVT_NOTIF_GPU_POST_RESET
+    RSMI_EVT_NOTIF_THERMAL_THROTTLE = 2
+    RSMI_EVT_NOTIF_GPU_PRE_RESET = 3
+    RSMI_EVT_NOTIF_GPU_POST_RESET = 4
+    RSMI_EVT_NOTIF_RING_HANG = 5
+    RSMI_EVT_NOTIF_LAST = RSMI_EVT_NOTIF_RING_HANG
 
 
 class rsmi_voltage_metric_t(c_int):
@@ -613,19 +583,19 @@ class rsmi_func_id_value_t(Union):
 
 class rsmi_compute_partition_type_t(c_int):
     RSMI_COMPUTE_PARTITION_INVALID = 0
-    RSMI_COMPUTE_PARTITION_CPX = 1
-    RSMI_COMPUTE_PARTITION_SPX = 2
-    RSMI_COMPUTE_PARTITION_DPX = 3
-    RSMI_COMPUTE_PARTITION_TPX = 4
-    RSMI_COMPUTE_PARTITION_QPX = 5
+    RSMI_COMPUTE_PARTITION_SPX = 1
+    RSMI_COMPUTE_PARTITION_DPX = 2
+    RSMI_COMPUTE_PARTITION_TPX = 3
+    RSMI_COMPUTE_PARTITION_QPX = 4
+    RSMI_COMPUTE_PARTITION_CPX = 5
 
 rsmi_compute_partition_type_dict = {
     #'RSMI_COMPUTE_PARTITION_INVALID': 0,
-    'CPX': 1,
-    'SPX': 2,
-    'DPX': 3,
-    'TPX': 4,
-    'QPX': 5
+    'SPX': 1,
+    'DPX': 2,
+    'TPX': 3,
+    'QPX': 4,
+    'CPX': 5,
 }
 
 rsmi_compute_partition_type = rsmi_compute_partition_type_t
@@ -634,7 +604,7 @@ rsmi_compute_partition_type = rsmi_compute_partition_type_t
 # Usage example to get corresponding names:
 # compute_partition_type_l[rsmi_compute_partition_type_t.RSMI_COMPUTE_PARTITION_CPX]
 # will return string 'CPX'
-compute_partition_type_l = ['CPX', 'SPX', 'DPX', 'TPX', 'QPX']
+compute_partition_type_l = ['SPX', 'DPX', 'TPX', 'QPX', 'CPX']
 
 class rsmi_memory_partition_type_t(c_int):
     RSMI_MEMORY_PARTITION_UNKNOWN = 0
@@ -672,3 +642,102 @@ rsmi_power_type_dict = {
     1: 'CURRENT SOCKET',
     0xFFFFFFFF: 'INVALID_POWER_TYPE'
 }
+
+class metrics_table_header_t(Structure):
+    pass
+
+# metrics_table_header_t._pack_ = 1 # source:False
+metrics_table_header_t._fields_ = [
+    ('structure_size', c_uint16),
+    ('format_revision', c_uint8),
+    ('content_revision', c_uint8),
+]
+amd_metrics_table_header_t = metrics_table_header_t
+
+class amdgpu_xcp_metrics_t(Structure):
+    pass
+
+# amdgpu_xcp_metrics_t._pack_ = 1 # source:False
+amdgpu_xcp_metrics_t._fields_ = [
+    ('gfx_busy_inst', c_uint32 * 8),
+    ('jpeg_busy', c_uint16 * 32),
+    ('vcn_busy', c_uint16 * 4),
+    ('gfx_busy_acc', c_uint64 * 8),
+]
+xcp_stats_t = amdgpu_xcp_metrics_t
+
+class rsmi_gpu_metrics_t(Structure):
+    pass
+
+
+# rsmi_gpu_metrics_t._pack_ = 1  # source:False
+rsmi_gpu_metrics_t._fields_ = [
+    ('common_header', amd_metrics_table_header_t),
+    ('temperature_edge', c_uint16),
+    ('temperature_hotspot', c_uint16),
+    ('temperature_mem', c_uint16),
+    ('temperature_vrgfx', c_uint16),
+    ('temperature_vrsoc', c_uint16),
+    ('temperature_vrmem', c_uint16),
+    ('average_gfx_activity', c_uint16),
+    ('average_umc_activity', c_uint16),
+    ('average_mm_activity', c_uint16),
+    ('average_socket_power', c_uint16),
+    ('energy_accumulator', c_uint64),
+    ('system_clock_counter', c_uint64),
+    ('average_gfxclk_frequency', c_uint16),
+    ('average_socclk_frequency', c_uint16),
+    ('average_uclk_frequency', c_uint16),
+    ('average_vclk0_frequency', c_uint16),
+    ('average_dclk0_frequency', c_uint16),
+    ('average_vclk1_frequency', c_uint16),
+    ('average_dclk1_frequency', c_uint16),
+    ('current_gfxclk', c_uint16),
+    ('current_socclk', c_uint16),
+    ('current_uclk', c_uint16),
+    ('current_vclk0', c_uint16),
+    ('current_dclk0', c_uint16),
+    ('current_vclk1', c_uint16),
+    ('current_dclk1', c_uint16),
+    ('throttle_status', c_uint32),
+    ('current_fan_speed', c_uint16),
+    ('pcie_link_width', c_uint16),
+    ('pcie_link_speed', c_uint16),
+    ('gfx_activity_acc', c_uint32),
+    ('mem_activity_acc', c_uint32),
+    ('temperature_hbm', c_uint16 * 4),
+    ('firmware_timestamp', c_uint64),
+    ('voltage_soc', c_uint16),
+    ('voltage_gfx', c_uint16),
+    ('voltage_mem', c_uint16),
+    ('indep_throttle_status', c_uint64),
+    ('current_socket_power', c_uint16),
+    ('vcn_activity', c_uint16 * 4),
+    ('gfxclk_lock_status', c_uint32),
+    ('xgmi_link_width', c_uint16),
+    ('xgmi_link_speed', c_uint16),
+    ('pcie_bandwidth_acc', c_uint64),
+    ('pcie_bandwidth_inst', c_uint64),
+    ('pcie_l0_to_recov_count_acc', c_uint64),
+    ('pcie_replay_count_acc', c_uint64),
+    ('pcie_replay_rover_count_acc', c_uint64),
+    ('xgmi_read_data_acc', c_uint64 * 8),
+    ('xgmi_write_data_acc', c_uint64 * 8),
+    ('current_gfxclks', c_uint16 * 8),
+    ('current_socclks', c_uint16 * 4),
+    ('current_vclk0s', c_uint16 * 4),
+    ('current_dclk0s', c_uint16 * 4),
+    ('jpeg_activity', c_uint16 * 32),
+    ('pcie_nak_sent_count_acc', c_uint32),
+    ('pcie_nak_rcvd_count_acc', c_uint32),
+    ('accumulation_counter', c_uint64),
+    ('prochot_residency_acc', c_uint64),
+    ('ppt_residency_acc', c_uint64),
+    ('socket_thm_residency_acc', c_uint64),
+    ('vr_thm_residency_acc', c_uint64),
+    ('hbm_thm_residency_acc', c_uint64),
+    ('num_partition', c_uint16),
+    ('xcp_stats', xcp_stats_t * 8),
+    ('pcie_lc_perf_other_end_recovery', c_uint32),
+]
+amdsmi_gpu_metrics_t = rsmi_gpu_metrics_t
