@@ -235,15 +235,7 @@ RocmSMI::Initialize(uint64_t flags) {
   int i_ret;
   std::ostringstream ss;
 
-  LOG_ALWAYS("=============== ROCM SMI initialize ================");
-  ROCmLogging::Logger::getInstance()->enableAllLogLevels();
-  // Leaving below to allow developers to check current log settings
-  // std::string logSettings = Logger::getInstance()->getLogSettings();
-  // std::cout << "Current log settings:\n" << logSettings << std::endl;
 
-  if (ROCmLogging::Logger::getInstance()->isLoggerEnabled()) {
-    logSystemDetails();
-  }
 
   assert(ref_count_ == 1);
   if (ref_count_ != 1) {
@@ -259,6 +251,15 @@ RocmSMI::Initialize(uint64_t flags) {
   // To help debug env variable issues
   // debugRSMIEnvVarInfo();
 
+  if (ROCmLogging::Logger::getInstance()->isLoggerEnabled()) {
+    ROCmLogging::Logger::getInstance()->enableAllLogLevels();
+    LOG_ALWAYS("=============== ROCM SMI initialize ================");
+    logSystemDetails();
+  }
+  // Leaving below to allow developers to check current log settings
+  // std::string logSettings = ROCmLogging::Logger::getInstance()->getLogSettings();
+  // std::cout << "Current log settings:\n" << logSettings << std::endl;
+
   while (!std::string(kAMDMonitorTypes[i]).empty()) {
       amd_monitor_types_.insert(kAMDMonitorTypes[i]);
       ++i;
@@ -566,9 +567,9 @@ std::string RocmSMI::getRSMIEnvVarInfo(void) {
   for (auto it=env_vars_.enum_overrides.begin();
        it != env_vars_.enum_overrides.end(); ++it) {
     DevInfoTypes type = static_cast<DevInfoTypes>(*it);
-    ss << (std::to_string(*it) + " (" + Device::devInfoTypesStrings.at(type) + ")");
+    ss << (std::to_string(*it) + " (" + Device::get_type_string(type) + ")");
     auto temp_it = it;
-    if(++temp_it != env_vars_.enum_overrides.end()) {
+    if (++temp_it != env_vars_.enum_overrides.end()) {
       ss << ", ";
     }
   }
@@ -713,6 +714,8 @@ static bool isAMDGPU(std::string dev_path) {
 uint32_t RocmSMI::DiscoverAmdgpuDevices(void) {
   std::string err_msg;
   uint32_t count = 0;
+  int32_t cardId = 0;
+  int32_t max_cardId = -1;
   std::ostringstream ss;
 
   // If this gets called more than once, clear previous findings.
@@ -736,6 +739,9 @@ uint32_t RocmSMI::DiscoverAmdgpuDevices(void) {
         if ((strcmp(dentry->d_name, ".") == 0) ||
            (strcmp(dentry->d_name, "..") == 0))
            continue;
+        sscanf(&dentry->d_name[strlen(kDeviceNamePrefix)], "%d", &cardId);
+        if (cardId > max_cardId)
+          max_cardId = cardId;
         count++;
     }
     dentry = readdir(drm_dir);
@@ -818,7 +824,7 @@ uint32_t RocmSMI::DiscoverAmdgpuDevices(void) {
 
   uint32_t cardAdded = 0;
   // Discover all root cards & gpu partitions associated with each
-  for (uint32_t cardId = 0; cardId < count; cardId++) {
+  for (uint32_t cardId = 0; cardId <= max_cardId; cardId++) {
     std::string path = kPathDRMRoot;
     path += "/card";
     path += std::to_string(cardId);
@@ -858,6 +864,15 @@ uint32_t RocmSMI::DiscoverAmdgpuDevices(void) {
            << " BDF = " << std::to_string(primaryBdfId)
            << " (" << print_int_as_hex(primaryBdfId) << ")";
         LOG_DEBUG(ss);
+        if (doesDeviceSupportPartitions && strCompPartition != "SPX"
+           && i->second.s_partition_id == 0) {
+          i->second.s_partition_id = i->second.s_function;
+          ss << __PRETTY_FUNCTION__ << " | (secondary node add) fall back - "
+            << "detected !SPX && partition_id == 0"
+            << "; function = " << std::to_string(i->second.s_function)
+            << "; partition_id = " << std::to_string(i->second.s_partition_id);
+          LOG_DEBUG(ss);
+        }
         ss << __PRETTY_FUNCTION__
            << " | (secondary node add) B4 AddToDeviceList() -->"
            << "\n[node_id = " << std::to_string(i->second.s_node_id)
@@ -876,6 +891,15 @@ uint32_t RocmSMI::DiscoverAmdgpuDevices(void) {
       } else {
         ss << __PRETTY_FUNCTION__ << " | primary node add ; "
            << " BDF = " << std::to_string(UINT64_MAX);
+        if (doesDeviceSupportPartitions && strCompPartition != "SPX"
+           && i->second.s_partition_id == 0) {
+          i->second.s_partition_id = i->second.s_function;
+          ss << __PRETTY_FUNCTION__ << " | (primary node add) fall back - "
+            << "detected !SPX && partition_id == 0"
+            << "; function = " << std::to_string(i->second.s_function)
+            << "; partition_id = " << std::to_string(i->second.s_partition_id);
+          LOG_DEBUG(ss);
+        }
         LOG_DEBUG(ss);
         ss << __PRETTY_FUNCTION__
            << " | (primary node add) After AddToDeviceList() -->"
@@ -1005,6 +1029,15 @@ uint32_t RocmSMI::DiscoverAmdgpuDevices(void) {
            << " BDF = " << std::to_string(myBdfId)
            << " (" << print_int_as_hex(myBdfId) << ")";
         LOG_DEBUG(ss);
+        if (doesDeviceSupportPartitions && strCompPartition != "SPX"
+           && it->second.s_partition_id == 0) {
+          it->second.s_partition_id = it->second.s_function;
+          ss << __PRETTY_FUNCTION__ << " | (secondary node add #2) fall back - "
+            << "detected !SPX && partition_id == 0"
+            << "; function = " << std::to_string(it->second.s_function)
+            << "; partition_id = " << std::to_string(it->second.s_partition_id);
+          LOG_DEBUG(ss);
+        }
         ss << __PRETTY_FUNCTION__
            << " | (secondary node add #2) B4 AddToDeviceList() -->"
            << "\n[node_id = " << std::to_string(it->second.s_node_id)
